from http import HTTPStatus
from re import Pattern
from typing import Callable, Optional, Union

from galileo_core.constants.request_method import RequestMethod
from galileo_core.helpers.dependencies import is_dependency_available

if is_dependency_available("pytest") and is_dependency_available("respx"):
    from pytest import fixture
    from respx import MockRouter, Route

    @fixture
    def mock_request(respx_mock: MockRouter) -> Callable:
        def curry(
            method: RequestMethod,
            path: Union[str, Pattern],
            params: Optional[dict] = None,
            status_code: int = HTTPStatus.OK,
            json: Optional[dict] = None,
            text: Optional[str] = None,
        ) -> Route:
            if isinstance(path, str):
                path = "/" + path
            if not text and not json:
                json = {}
            return respx_mock.request(
                method=method,
                # Match any URL that ends with the path.
                url=path,
                params=params,
            ).respond(status_code=status_code, text=text, json=json)

        return curry



from enum import Enum
from typing import Literal


class ScorerType(str, Enum):
    luna = "luna"
    plus = "plus"


PlusScorerType = Literal[ScorerType.plus]
LunaOrPlusScorerType = Literal[ScorerType.luna, ScorerType.plus]



from typing import List, Optional

from pydantic import UUID4

from galileo_core.constants.request_method import RequestMethod
from galileo_core.constants.routes import Routes
from galileo_core.helpers.logger import logger
from galileo_core.schemas.base_config import BaseConfig
from galileo_core.schemas.core.project import CreateProjectRequest, ProjectResponse, ProjectType


def create_project(request: CreateProjectRequest) -> ProjectResponse:
    """
    Create a project.

    Parameters
    ----------
    request : CreateProjectRequest
        Request object for creating a project.

    Returns
    -------
    ProjectResponse
        Response object for the created project.
    """
    config = BaseConfig.get()
    existing_project = get_project_from_name(project_name=request.name, raise_if_missing=False)
    if existing_project:
        logger.debug(f"Project {request.name} already exists, using it.")
        project_response = existing_project
    else:
        logger.debug(f"Creating project {request.name}...")
        response_dict = config.api_client.request(
            RequestMethod.POST, Routes.projects, json=request.model_dump(mode="json")
        )
        project_response = ProjectResponse.model_validate(response_dict)
        logger.debug(f"Created project with name {project_response.name}, ID {project_response.id}.")
    return project_response


def get_project(
    project_id: Optional[UUID4] = None,
    project_name: Optional[str] = None,
    project_type: Optional[ProjectType] = None,
    raise_if_missing: bool = True,
) -> Optional[ProjectResponse]:
    """
    Get a project by either ID or name.

    If both project_id and project_name are provided, project_id will take precedence.

    For cases when the project name is if the project_type is provided, it will be used
    to filter the projects by type if the project_name is provided. If raise_if_missing
    is True, a ValueError will be raised if the project is not found. This is useful
    when the project is expected to exist, and the absence of the project would be an error.

    Parameters
    ----------
    project_id : Optional[UUID4], optional
        Project ID, by default None.
    project_name : Optional[str], optional
        Project name, by default None.
    project_type : Optional[ProjectType], optional
        Project type, by default None.
    raise_if_missing : bool, optional
        Raise an error if the project is not found, by default True.

    Returns
    -------
    Optional[ProjectResponse]
        Project response object if the project is found, None otherwise.

    Raises
    ------
    ValueError
        If neither project_id nor project_name is provided.
    """
    if project_id:
        return get_project_from_id(project_id=project_id)
    elif project_name:
        return get_project_from_name(
            project_name=project_name,
            project_type=project_type,
            raise_if_missing=raise_if_missing,
        )
    else:
        raise ValueError("Either project_id or project_name must be provided.")


def get_project_from_id(project_id: UUID4) -> ProjectResponse:
    """
    Given a project ID, get the project.

    Parameters
    ----------
    project_id : UUID4
        Project ID.

    Returns
    -------
    ProjectResponse
        Project response object.
    """
    config = BaseConfig.get()
    response_dict = config.api_client.request(RequestMethod.GET, Routes.project.format(project_id=project_id))
    project = ProjectResponse.model_validate(response_dict)
    logger.debug(f"Got project with name {project.name}, ID {project.id}.")
    return project


def get_projects(project_type: Optional[ProjectType] = None) -> List[ProjectResponse]:
    """
    Given a project type, returns all projects of that type that the user has access to.

    Parameters
    ----------
    project_type : ProjectType
        The type of projects to get.

    Returns
    -------
    List[ProjectResponse]
        A list of projects of the given type.
    """
    config = BaseConfig.get()
    params = dict(project_type=project_type.value) if project_type else {}
    projects = [
        ProjectResponse.model_validate(proj)
        for proj in config.api_client.request(RequestMethod.GET, Routes.projects, params=params)
    ]
    logger.debug(f"Got {len(projects)} projects of type {project_type}.")
    return projects


def get_project_from_name(
    project_name: str,
    project_type: Optional[ProjectType] = None,
    raise_if_missing: bool = True,
) -> Optional[ProjectResponse]:
    """
    Get a project by name.

    Parameters
    ----------
    project_name : str
        Name of the project.
    project_type : Optional[ProjectType], optional
        Type of the project to filter by, by default None.
    raise_if_missing : bool, optional
        Raise an error if the proejct is not found, by default True.

    Returns
    -------
    Optional[ProjectResponse]
        Project response object if the project is found, None otherwise.

    Raises
    ------
    ValueError
        If the project is not found and raise_if_missing is True.
    """
    config = BaseConfig.get()
    params = dict(project_name=project_name)
    if project_type:
        params["project_type"] = project_type
    projects = [
        ProjectResponse.model_validate(proj)
        for proj in config.api_client.request(RequestMethod.GET, Routes.projects, params=params)
    ]
    if raise_if_missing and len(projects) == 0:
        raise ValueError(f"Project {project_name} does not exist.")
    elif len(projects) > 0:
        project_response = projects[0]
        logger.debug(f"Got project with name {project_response.name}, with ID {project_response.id}.")
        return project_response
    else:
        return None



from pydantic import UUID4, BaseModel

from galileo_core.schemas.core.collaboration_role import CollaboratorRole


class UserProjectCollaboratorRequest(BaseModel):
    role: CollaboratorRole
    user_id: UUID4


class UserProjectCollaboratorResponse(UserProjectCollaboratorRequest):
    user_id: UUID4
    email: str



from typing import Optional

from pydantic import BaseModel, ConfigDict, Field, ValidationInfo, field_validator

from galileo_core.schemas.shared.scorers.scorer_name import ScorerName
from galileo_core.schemas.shared.scorers.scorer_type import LunaOrPlusScorerType, PlusScorerType, ScorerType


class BaseScorerConfig(BaseModel):
    name: str = Field(..., description="Name of the scorer to enable.")


class GalileoScorerConfig(BaseScorerConfig):
    name: ScorerName = Field(..., description="Name of the scorer to enable.")


class LunaOrPlusScorerTypeConfig(BaseModel):
    type: LunaOrPlusScorerType = ScorerType.luna


class PlusScorerTypeConfig(BaseModel):
    type: PlusScorerType = ScorerType.plus


class PlusScorerConfig(GalileoScorerConfig):
    type: ScorerType
    model_name: Optional[str] = Field(default=None, description="Alias of the model to use for the scorer.")

    model_config = ConfigDict(
        # Avoid Pydantic's protected namespace warning since we want to use
        # `model_alias` as a field name.
        protected_namespaces=(),
    )

    @field_validator("model_name", mode="before")
    def validate_model_alias(cls, value: Optional[str], info: ValidationInfo) -> Optional[str]:
        name = info.data["name"]
        scorer_type = info.data.get("type", ScorerType.luna)
        if value is not None and scorer_type != ScorerType.plus:
            raise ValueError(f"Model alias is not allowed for {name} scorer if type is not set to plus.")
        return value


class PlusScorerWithNumJudgesConfig(PlusScorerConfig):
    num_judges: Optional[int] = Field(default=None, ge=1, le=10, description="Number of judges for the scorer.")

    @field_validator("num_judges", mode="before")
    def validate_num_judges(cls, value: Optional[int], info: ValidationInfo) -> Optional[int]:
        name = info.data["name"]
        scorer_type = info.data.get("type", ScorerType.luna)
        if value is not None and scorer_type != ScorerType.plus:
            raise ValueError(f"Number of judges is not allowed for {name} scorer if type is not set to plus.")
        return value


class RegisteredScorerConfig(BaseScorerConfig): ...


class GeneratedScorerConfig(BaseScorerConfig): ...



from importlib.util import find_spec


def is_dependency_available(name: str) -> bool:
    """
    Check if a dependency is available.

    Parameters
    ----------
    name : str
        The name of the dependency to check.

    Returns
    -------
    bool
        True if the dependency is available, False otherwise.
    """
    return find_spec(name) is not None



from typing import Literal, Union

from pydantic import Field
from typing_extensions import Annotated

from galileo_core.schemas.shared.scorers.base_configs import (
    GalileoScorerConfig,
    LunaOrPlusScorerTypeConfig,
    PlusScorerConfig,
    PlusScorerTypeConfig,
    PlusScorerWithNumJudgesConfig,
)
from galileo_core.schemas.shared.scorers.scorer_name import ScorerName


class BleuScorer(GalileoScorerConfig):
    name: Literal[ScorerName.bleu] = ScorerName.bleu


class ChunkAttributionUtilizationScorer(LunaOrPlusScorerTypeConfig, PlusScorerConfig):
    name: Literal[ScorerName.chunk_attribution_utilization] = ScorerName.chunk_attribution_utilization


class CompletenessScorer(LunaOrPlusScorerTypeConfig, PlusScorerWithNumJudgesConfig):
    name: Literal[ScorerName.completeness] = ScorerName.completeness


class ContextAdherenceScorer(LunaOrPlusScorerTypeConfig, PlusScorerWithNumJudgesConfig):
    name: Literal[ScorerName.context_adherence] = ScorerName.context_adherence


class ContextRelevanceScorer(GalileoScorerConfig):
    name: Literal[ScorerName.context_relevance] = ScorerName.context_relevance


class CorrectnessScorer(PlusScorerTypeConfig, PlusScorerWithNumJudgesConfig):
    name: Literal[ScorerName.correctness] = ScorerName.correctness


class InputPIIScorer(GalileoScorerConfig):
    name: Literal[ScorerName.input_pii] = ScorerName.input_pii


class InputSexistScorer(GalileoScorerConfig):
    name: Literal[ScorerName.input_sexist] = ScorerName.input_sexist


class InputToneScorer(GalileoScorerConfig):
    name: Literal[ScorerName.input_tone] = ScorerName.input_tone


class InputToxicityScorer(GalileoScorerConfig):
    name: Literal[ScorerName.input_toxicity] = ScorerName.input_toxicity


class InstructionAdherenceScorer(PlusScorerTypeConfig, PlusScorerWithNumJudgesConfig):
    name: Literal[ScorerName.instruction_adherence] = ScorerName.instruction_adherence


class OutputPIIScorer(GalileoScorerConfig):
    name: Literal[ScorerName.output_pii] = ScorerName.output_pii


class OutputSexistScorer(GalileoScorerConfig):
    name: Literal[ScorerName.output_sexist] = ScorerName.output_sexist


class OutputToneScorer(GalileoScorerConfig):
    name: Literal[ScorerName.output_tone] = ScorerName.output_tone


class OutputToxicityScorer(GalileoScorerConfig):
    name: Literal[ScorerName.output_toxicity] = ScorerName.output_toxicity


class PromptInjectionScorer(GalileoScorerConfig):
    name: Literal[ScorerName.prompt_injection] = ScorerName.prompt_injection


class PromptPerplexityScorer(GalileoScorerConfig):
    name: Literal[ScorerName.prompt_perplexity] = ScorerName.prompt_perplexity


class RougeScorer(GalileoScorerConfig):
    name: Literal[ScorerName.rouge] = ScorerName.rouge


class ToolSelectionScorer(PlusScorerTypeConfig, PlusScorerWithNumJudgesConfig):
    name: Literal[ScorerName.tool_selection] = ScorerName.tool_selection


class UncertaintyScorer(GalileoScorerConfig):
    name: Literal[ScorerName.uncertainty] = ScorerName.uncertainty


GalileoScorer = Annotated[
    Union[
        BleuScorer,
        ChunkAttributionUtilizationScorer,
        CompletenessScorer,
        ContextAdherenceScorer,
        ContextRelevanceScorer,
        CorrectnessScorer,
        InputPIIScorer,
        InputSexistScorer,
        InputToneScorer,
        InputToxicityScorer,
        InstructionAdherenceScorer,
        OutputPIIScorer,
        OutputSexistScorer,
        OutputToneScorer,
        OutputToxicityScorer,
        PromptInjectionScorer,
        PromptPerplexityScorer,
        RougeScorer,
        UncertaintyScorer,
    ],
    Field(discriminator="name"),
]



from typing import Union

from pydantic import BaseModel

from galileo_core.schemas.shared.message_role import MessageRole


class Message(BaseModel):
    content: str
    role: Union[str, MessageRole]

    @property
    def message(self) -> str:
        role = self.role.value if isinstance(self.role, MessageRole) else self.role
        return f"{role}: {self.content}"
